from relevanceai import vector_tools
from relevanceai.http_client import Client

# Import useful utility if possible as well
try:
    from jsonshower import show_json
except ModuleNotFoundError:
    pass

__version__ = "0.21.1"
